import ssl
import typing

import aiohttp
import requests

CurrentSession = typing.TypeVar("CurrentSession", bound=typing.Union[aiohttp.ClientSession, requests.Session])


def get_session(session_type: typing.Type[CurrentSession], **kwargs) -> CurrentSession:
    if isinstance(session_type, aiohttp.ClientSession):
        return aiohttp.ClientSession(
            connector=aiohttp.TCPConnector(ssl=ssl.SSLContext()),
            skip_auto_headers={"User-Agent"},
            raise_for_status=True,
            **kwargs
        )
    else:
        return requests.Session()


# Async function to download file
async def coroutine_download(url: str) -> bytes:
    current_session = get_session(aiohttp.ClientSession)
    async with current_session.get(url) as file:
        return await file.read()


# Sync function to download file
def download(url: str) -> bytes:
    current_session = get_session(requests.Session)
    with current_session.get(url, allow_redirects=True) as file:
        return file.content



import datetime
import io
import os
import typing

from PIL import Image
from PIL import ImageDraw
from PIL import ImageFont

from .generate_utils import (Quote,
                             prepare_round_the_resulting_image,
                             prepare_mask,
                             prepare_size_by_text_length,
                             prepare_coordinates_by_text_length)

from .downloader import coroutine_download
from .downloader import download

ImageType = typing.TypeVar("ImageType",
                           bound=typing.Union[Image.Image, bytes, io.BytesIO, str])


def open_image_with_bytes(bytes_argument: bytes) -> Image.Image:
    return Image.open(io.BytesIO(bytes_argument))


class QuoteManager:

    def __init__(self,
                 fullname: str = "Igor Nezhivykh",
                 text: str = "Hello! This is the <Quote Manager> library, the version of this library is 1.0.0."
                             " Please put the new argument in the <<text>> object",
                 date: str = "%d.%m.%Y в %H:%M:%S",
                 by_project: typing.Union[str, bool, None] = "by https://github.com/ymoth",
                 path_to_dependencies: str = "default_dependencies",
                 background_image: ImageType = "background.jpg",
                 avatar_image: ImageType = "default_avatar.jpg",
                 default_font: typing.Union[str, None] = "Rubik-Regular.ttf",
                 string_limit: int = 300,
                 string_line_limit: int = 30,
                 **kwargs):

        # Default arguments
        self._path_to_dependencies = path_to_dependencies
        self._fullname = fullname
        self._text = text
        self._date = date
        self._by_project = by_project

        # Optional arguments
        self._background_image = background_image
        self._avatar_image = avatar_image
        self._default_font = default_font
        self._string_line_limit = string_line_limit
        self._string_limit = string_limit

        # Setting sizes options
        self._size_avatar = kwargs.get("size_avatar", (300, 300))
        self._size_background = kwargs.get("size_background", (0, 0, 1280, 720))

        self._coordinates_avatar = kwargs.get("coordinates_avatar", (50, 210))
        self._coordinates_fullname = kwargs.get("coordinates_name", (70, 520))
        self._coordinates_date = kwargs.get("coordinates_date", (70, 545))
        self._coordinates_by_project = kwargs.get("coordinates_by_project", (70, 570))

        # Fonts
        self._fullname_font = kwargs.get("fullname_font", self._default_font)
        self._date_font = kwargs.get("date_font", self._default_font)
        self._main_text_font = kwargs.get('main_text_font', "zero_5.ttf")
        self._by_project_font = kwargs.get("by_project_font", self._default_font)

    def _prepare_quote_by_avatar_and_by_background(self, background: Image, avatar: Image) -> Image:
        """Создание самой цитаты, в которую попадают два параметра:
        background: Image, avatar: Image"""

        # Интеграция Image-объекта на аватарку для использования наложения
        avatar = prepare_round_the_resulting_image(avatar, self._size_avatar)
        avatar.putalpha(prepare_mask(self._size_avatar, 4))

        # Обрезка фона до нужно ширины, и наложение маски на аватарку.
        crop_background_image = background.crop(self._size_background)
        image_mask = Image.new("L", avatar.size, 0)

        # Округление аватара и наложение на слой бэкграунда
        draw = ImageDraw.Draw(image_mask)
        draw.ellipse((0, 0, *self._size_avatar), fill=255)
        crop_background_image.paste(avatar, self._coordinates_avatar, image_mask)

        prepare_image_draw = ImageDraw.Draw(crop_background_image)
        prepare_image_draw.text(self._coordinates_fullname, self._fullname,
                                font=self._generate_font(20))

        # Проверка на дату и добавление данных в цитату
        if self._date is not None:
            prepare_image_draw.text(self._coordinates_date,
                                    self._crop_message(datetime.datetime.now().strftime(self._date)),
                                    font=self._generate_font(20, font=self._date_font))

        # Проверка на проект и добавление данных в цитату
        if self._by_project is not None:
            prepare_image_draw.text(self._coordinates_by_project, self._crop_message(self._by_project, limit=30, ),
                                    font=self._generate_font(20, font=self._by_project_font))

        # Подставка текста, автоматическая реализация сайзинга и обрезания сообщения
        # За сайзинг отвечает функция generate_utils.prepare_size_by_text_length:55
        # За центрирование(координаты) отвечает функция prepare_coordinates_by_text_length:69
        prepare_image_draw.text(prepare_coordinates_by_text_length(self._text),
                                self._crop_message(self._text),
                                font=ImageFont.truetype(self._path_to_dependencies + "//" + self._main_text_font,
                                                        size=prepare_size_by_text_length(self._main_text_font),
                                                        encoding="utf-8"))

        # Сохранение в io.Bytes()
        img_byte_arr = io.BytesIO()
        crop_background_image.save(img_byte_arr, format='PNG')

        # Возвращаемый объект, который попадёт в датакласс `Quote`
        return img_byte_arr

    def _generate_font(self, font_size: int = None, font: str = None) -> Image:
        font_size = 40 if font_size is None else font_size
        font = font if font else self._default_font
        if font not in os.listdir(self._path_to_dependencies) or self._default_font is None:
            font = os.getcwd() + "/default_dependencies/Rubik-Regular.ttf"
        else:
            font = self._default_font

        return ImageFont.truetype(
            self._path_to_dependencies + "//" + font,
            size=font_size, encoding="utf-8"
        )

    def _crop_message(self, message: str, limit: int = None) -> str:
        limit = self._string_line_limit if limit is None else limit
        return "\n".join(
            [message[x:x + limit].replace("\n", " ") for x in range(0, len(message), limit)]
        )

    async def coroutine_create(self) -> Quote:
        background = await self._async_getting_image_element(self._background_image)
        avatar = await self._async_getting_image_element(self._background_image)
        return Quote(self._prepare_quote_by_avatar_and_by_background(background=background, avatar=avatar))

    def sync_create(self) -> Quote:
        background = self._sync_getting_image_element(self._background_image)
        avatar = self._sync_getting_image_element(self._avatar_image)
        return Quote(self._prepare_quote_by_avatar_and_by_background(background=background, avatar=avatar))

    async def _async_getting_image_element(self, attribute_image: ImageType) -> Image.Image:
        if isinstance(attribute_image, str):
            if "https://" in attribute_image:
                return open_image_with_bytes(await coroutine_download(attribute_image))
            else:
                return Image.open(self._path_to_dependencies + "//" + attribute_image)
        return Image.open(io.BytesIO(attribute_image) if isinstance(attribute_image, bytes) else attribute_image)

    def _sync_getting_image_element(self, attribute_image: ImageType) -> Image.Image:
        if isinstance(attribute_image, str):
            if "https://" in attribute_image:
                return open_image_with_bytes(download(attribute_image))
            else:
                return Image.open(self._path_to_dependencies + "//" + attribute_image)
        return Image.open(io.BytesIO(attribute_image) if isinstance(attribute_image, bytes) else attribute_image)



import dataclasses
import io
import os
import typing

from PIL import ImageDraw
from PIL import Image

type_size = typing.Tuple[typing.Union[int, float], typing.Union[int, float]]


@dataclasses.dataclass
class Quote:
    _content: io.BytesIO

    @property
    def get_bytes_io(self) -> io.BytesIO:
        return self._content

    def save_image(self, path: str, image_name: str = None) -> None:
        Image.open(self._content).save(
            path + f"/quote_{os.urandom(3).hex() if not image_name else image_name}.jpg"
        )
        return None

    def show_image(self) -> None:
        Image.open(self._content).show()
        return None


@dataclasses.dataclass
class GenerateByLength:
    text: str


# Создание готовой макси для аватара, возвращает готовую маску.
def prepare_mask(sizing: type_size, alias_resize: int = 2) -> Image:
    mask = Image.new('L', (sizing[0] * alias_resize, sizing[1] * alias_resize), 0)
    ImageDraw.Draw(mask).ellipse((0, 0) + mask.size, fill=255)
    return mask.resize(sizing, Image.ANTIALIAS)


# Обрезает и масштабирует изображение под заданный размер.
def prepare_round_the_resulting_image(image: Image, sizing: type_size):
    weight, height = image.size
    result = weight / sizing[0] - height / sizing[1]
    if result > 0:
        image = image.crop(((weight - height) / 2, 0, (weight + height) / 2, height))
    elif result < 0:
        image = image.crop((0, (height - weight) / 2, weight, (height + weight) / 2))
    return image.resize(sizing, Image.ANTIALIAS)


# Данные функция используется исключительно для главного текста, который попадает на середину Image.
def prepare_size_by_text_length(text: str) -> int:
    """Обычная проверка на длину строки, максимальный аргумент = self.limit
    Возвращает size для ImageDraw.truetype"""
    length = len(text)
    if length <= 50:
        return 48
    if length <= 100:
        return 34
    if length <= 150:
        return 23
    return 15


# Данные функция используется исключительно для главного текста, который попадает на середину Image.
def prepare_coordinates_by_text_length(text: str) -> type_size:
    """Обычная проверка на длину строки, максимальный аргумент: 300
    Возвращает координаты для Image.draw"""
    length = len(text[:300])
    if length <= 50:
        return 450, 300
    elif length <= 100:
        return 400, 210
    return 370, 180



from setuptools import setup

"""
:authors: ymoth
:copyright: (c) 2022 ymoth
"""

version = "1.1.3"

setup(
    name="quote_depencives",
    version=version,

    author="ymoth",
    author_email="tophanbig@gmail.com",
    description="Easy create quote image by agrmunets in class options",

    url="https://github.com/ymoth/quote-manager",
    packages=["quote_depencives", "quote_depencives/default_dependencies"],
    requires=["aiohttp", "requests", "Pillow"],
)
